/* Example of correlation between two columns within the same table.  Fixed by SQL Tuning advisor
and SQL profile.
*/

set linesize 1000
set pagesize 1000
set echo on
set termout on

spool test3.log

drop table TEST3;

create table TEST3 (A NUMBER,B NUMBER);

/* One million rows of 1,1.  One million rows of 2,2.  One row 1,2. */

declare
  i number;
begin
  i:=1;
  loop
    insert into TEST3 values (1,1);
    insert into TEST3 values (2,2);
    i:=i+1;
    exit when (i > 1000000);
    if (mod(i,10000)=0) then
      commit;
    end if;
  end loop;
  commit;
  insert into TEST3 values (1,2);
  commit;
end;
/

create index TEST3INDEX on TEST3(A,B); 

/* Gather compute stats with all histograms. */

execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'TEST3',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');

/* Remove any profile from previous run of this test. */

begin
  DBMS_SQLTUNE.DROP_SQL_PROFILE (name=>'my_sql_profile',ignore =>TRUE);
end;
/

/* Get plan without profile. */

explain plan into plan_table for 
select sum(a+b) 
from TEST3
where
a=1 and b=2;

set echo off
@utlxpls.sql
set echo on

/*  Show column statistics and histogram info */

select 
substr(column_name,1,1) "Column Name",
UTL_RAW.CAST_TO_NUMBER(low_value) Low,
UTL_RAW.CAST_TO_NUMBER(high_value) High,
num_distinct,num_buckets
from USER_TAB_COLUMNS where table_name='TEST3' and 
(column_name='A' or column_name='B')
order by column_name;

/* Show the number of rows in the table statistic */

select num_rows
from USER_TABLES 
where table_name='TEST3';

/* Show how long it takes to run the statement */

set timing on

select sum(a+b) 
from TEST3
where
a=1 and b=2;

set timing off

/* Use SQL Tuning advisor to create a SQL profile that will change the way the statement is
executed.  */

begin

/* Delete this task from previous runs of this script if it exists. */               
               
 DBMS_ADVISOR.DELETE_TASK (task_name   => 'my_sql_tuning_task');

end;
/

DECLARE
 my_task_name VARCHAR2(30);
 my_sqltext   CLOB;
BEGIN

/* SQL to be tuned */

 my_sqltext := 'select sum(a+b) '      ||
               'from TEST3 '           ||
               'where '                ||
               'a=1 and b=2';
             

/* Create and execute the tuning task. */               

 my_task_name := DBMS_SQLTUNE.CREATE_TUNING_TASK(
         sql_text    => my_sqltext,
         user_name   => NULL,
         scope       => 'COMPREHENSIVE',
         time_limit  => 600,
         task_name   => 'my_sql_tuning_task',
         description => 'Task to tune TEST3');
         
DBMS_SQLTUNE.EXECUTE_TUNING_TASK( task_name => 'my_sql_tuning_task' );

END;
/

/* This section of SQL outputs the report that came back from the SQL Tuning advisor.
Indicates a "SQL Profile Finding" which means you can change the plan by accepting the 
SQL profile.  */

SET LONG 1000
SET LONGCHUNKSIZE 1000
SET LINESIZE 100
SELECT DBMS_SQLTUNE.REPORT_TUNING_TASK( 'my_sql_tuning_task')
  FROM DUAL;

/* Accepts the SQL profile created by the SQL Tuning advisor task above. Now it will apply whenever the
exact same SQL statement is run.  */

DECLARE
 my_sqlprofile_name VARCHAR2(30);

BEGIN
my_sqlprofile_name := DBMS_SQLTUNE.ACCEPT_SQL_PROFILE ( 
    task_name => 'my_sql_tuning_task',
    name      => 'my_sql_profile');

END;
/

/* Check out the plan after accepting the new profile */

explain plan into plan_table for 
select sum(a+b) 
from TEST3
where
a=1 and b=2;

set echo off
@utlxpls.sql
set echo on

/* Show how long it takes to run the statement */

set timing on

select sum(a+b) 
from TEST3
where
a=1 and b=2;

set timing off

/* Show distribution of data in the table */

select a,b,count(*) from TEST3 group by a,b;

drop table TEST3;

spool off
