/* Example of correlation between columns in different tables. SQL Tuning advisor and 
SQL profile do not fix this one.
*/

set linesize 1000
set pagesize 1000
set echo on
set termout on

spool test4.log

/* Small table with just two rows which point to large table.
Idea is that where the NAME column='FEW' there are few records in the large table.
Where the NAME column='MANY' there are many records.  But the optimizer can't figure this out
given the statistics it has available. */

create table SMALL (NAME VARCHAR2(4),NUM NUMBER);
create table LARGE (NUM NUMBER);

insert into SMALL values ('MANY',1);
insert into SMALL values ('FEW',2);

commit;

/* Large table has 1,000,000 1's and one 2 */

declare
  i number;
begin
  i:=1;
  loop
    insert into LARGE values (1);
    i:=i+1;
    if (mod(i,10000)=0) then
      commit;
    end if;
    exit when (i > 1000000);
  end loop;
  commit;
  insert into LARGE values (2);
  commit;
end;
/

create index LARGEINDEX on LARGE(NUM); 

/* Gather compute stats with max number of histograms all columns.  */

execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'SMALL',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');
execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'LARGE',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');

/* Show plan.  Does an index scan even though only one row will be returned.  */

explain plan into plan_table for 
select B.NUM 
from SMALL A,LARGE B
where
a.NUM=B.NUM and
A.NAME='FEW';


set echo off
@utlxpls.sql
set echo on

/*  Show column statistics and histogram info */

select 
substr(UTL_RAW.CAST_TO_VARCHAR2(low_value),1,4) Low,
substr(UTL_RAW.CAST_TO_VARCHAR2(high_value),1,4) High,
num_distinct,num_buckets
from USER_TAB_COLUMNS where table_name='SMALL' and 
column_name='NAME';

select UTL_RAW.CAST_TO_NUMBER(low_value) Low,
UTL_RAW.CAST_TO_NUMBER(high_value) High,
num_distinct,num_buckets
from USER_TAB_COLUMNS where table_name='SMALL' and 
column_name='NUM';

select UTL_RAW.CAST_TO_NUMBER(low_value) Low,
UTL_RAW.CAST_TO_NUMBER(high_value) High,
num_distinct,num_buckets
from USER_TAB_COLUMNS where table_name='LARGE' and 
column_name='NUM';

/* Show the number of rows in the table statistics */

select num_rows
from USER_TABLES 
where table_name='SMALL';

select num_rows
from USER_TABLES 
where table_name='LARGE';

/* Try using SQL Tuning advisor just like we did in test3 script. */

begin

 DBMS_ADVISOR.DELETE_TASK (task_name   => 'my_sql_tuning_task');

end;
/

DECLARE
 my_task_name VARCHAR2(30);
 my_sqltext   CLOB;
BEGIN
 my_sqltext := 'select B.NUM '         ||
               'from SMALL A,LARGE B ' ||
               'where '                ||
               'a.NUM=B.NUM and '      ||
               'A.NAME=''FEW''';
               
 my_task_name := DBMS_SQLTUNE.CREATE_TUNING_TASK(
         sql_text    => my_sqltext,
         user_name   => NULL,
         scope       => 'COMPREHENSIVE',
         time_limit  => 600,
         task_name   => 'my_sql_tuning_task',
         description => 'Task to tune test6');
         
DBMS_SQLTUNE.EXECUTE_TUNING_TASK( task_name => 'my_sql_tuning_task' );

END;
/

/* Show output of SQL Tuning advisor.  It has no recommendations so you can't use a SQL profile */

SET LONG 1000
SET LONGCHUNKSIZE 1000
SET LINESIZE 100
SELECT DBMS_SQLTUNE.REPORT_TUNING_TASK( 'my_sql_tuning_task')
  FROM DUAL;

set timing on

/* Show how long it takes with the wrong plan. */

select B.NUM 
from SMALL A,LARGE B
where
a.NUM=B.NUM and
A.NAME='FEW';

/* Show how long it takes with the index being used correctly.  */

select /*+ INDEX(B LARGEINDEX) NO_INDEX_FFS(B LARGEINDEX)  */ B.NUM 
from SMALL A,LARGE B
where
a.NUM=B.NUM and
A.NAME='FEW';

set timing off

/* Show distribution of the data */

select NUM, count(*) from LARGE group by NUM;

drop table SMALL;
drop table LARGE;

/* Show how to change application to fix problem of correlation of columns between tables.  
Eliminated small table.  Broke large table into two.  Changed SQL.  */


create table LARGEA (NAME VARCHAR2(4),NUM NUMBER);
create table LARGEB (NAME VARCHAR2(4),NUM NUMBER);


declare
  i number;
begin
  i:=1;
  loop
    insert into LARGEA values ('MANY',1);
    i:=i+1;
    if (mod(i,10000)=0) then
      commit;
    end if;
    exit when (i > 1000000);
  end loop;
  commit;
  insert into LARGEB values ('FEW',2);
  commit;
end;
/

create index LARGEAINDEX on LARGEA(NAME); 
create index LARGEBINDEX on LARGEB(NAME); 

/* Gather compute stats with max number of histograms all columns.  */

execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'LARGEA',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');
execute DBMS_STATS.GATHER_TABLE_STATS (NULL,'LARGEB',cascade=>TRUE,estimate_percent=>NULL,method_opt=>'FOR ALL COLUMNS SIZE 254');

explain plan into plan_table for 
select NUM 
from (select * from largea
union
select * from largeb)
where
NAME='FEW';

set echo off
@utlxpls.sql
set echo on

set timing on

select NUM 
from (select * from largea
union
select * from largeb)
where
NAME='FEW';

set timing off

drop table LARGEA;
drop table LARGEB;

spool off
